@@ -2,12 +2,11 @@ package Rstats::Util;
 use strict;
 use warnings;
 
-require Rstats;
 use Scalar::Util ();
 use B ();
 use Carp 'croak';
 use Rstats::Func;
-use Rstats::ElementsFunc;
+use Rstats::ElementFunc;
 
 sub is_perl_number {
   my ($value) = @_;
@@ -45,7 +44,7 @@ sub looks_like_na {
   return if !defined $value || !CORE::length $value;
   
   if ($value eq 'NA') {
-    return Rstats::ElementsFunc::NA();
+    return Rstats::ElementFunc::NA();
   }
   else {
     return;
@@ -59,10 +58,10 @@ sub looks_like_logical {
   
   if ($value =~ /^(T|TRUE|F|FALSE)$/) {
     if ($value =~ /T/) {
-      return Rstats::ElementsFunc::logical(1);
+      return Rstats::ElementFunc::logical(1);
     }
     else {
-      return Rstats::ElementsFunc::logical(0);
+      return Rstats::ElementFunc::logical(0);
     }
   }
   else {
@@ -70,14 +69,27 @@ sub looks_like_logical {
   }
 }
 
-sub looks_like_double {
+sub looks_like_integer {
+  my $value = shift;
+  
+  return if !defined $value || !CORE::length $value;
+  
+  if ($value =~ /^[-+]?[0-9]+$/) {
+    return $value + 0;
+  }
+  else {
+    return;
+  }
+}
+
+sub looks_like_number {
   my $value = shift;
   
   return if !defined $value || !CORE::length $value;
   $value =~ s/^ +//;
   $value =~ s/ +$//;
   
-  if (Scalar::Util::looks_like_number($value)) {
+  if (Scalar::Util::looks_like_number $value) {
     return $value + 0;
   }
   else {
@@ -108,7 +120,7 @@ sub looks_like_complex {
     return;
   }
   
-  if (defined Rstats::Util::looks_like_double($re) && defined Rstats::Util::looks_like_double($im)) {
+  if (defined Rstats::Util::looks_like_number($re) && defined Rstats::Util::looks_like_number($im)) {
     return {re => $re + 0, im => $im + 0};
   }
   else {
@@ -123,10 +135,10 @@ sub parse_index {
   my @indexs;
   my @x2_dim;
 
-  if (ref $_indexs[0] && $_indexs[0]->is_array && $_indexs[0]->is_logical && @{$_indexs[0]->dim->decompose_elements} > 1) {
+  if (ref $_indexs[0] && $_indexs[0]->is_array && $_indexs[0]->is_logical && @{$_indexs[0]->dim->elements} > 1) {
     my $x2 = $_indexs[0];
     my $x2_dim_values = $x2->dim->values;
-    my $x2_elements = $x2->decompose_elements;
+    my $x2_elements = $x2->elements;
     my $poss = [];
     for (my $i = 0; $i < @$x2_elements; $i++) {
       next unless $x2_elements->[$i];
@@ -187,9 +199,8 @@ sub parse_index {
       }
       elsif ($index->is_logical) {
         my $index_values_new = [];
-        my $index_elements = $index->decompose_elements;
         for (my $i = 0; $i < @{$index->values}; $i++) {
-          push @$index_values_new, $i + 1 if $index_elements->[$i];
+          push @$index_values_new, $i + 1 if $index->elements->[$i];
         }
         $index = Rstats::Func::array($index_values_new);
       }
@@ -204,7 +215,7 @@ sub parse_index {
 
       push @indexs, $index;
 
-      my $count = $index->elements->length_value;
+      my $count = @{$index->elements};
       push @x2_dim, $count unless $count == 1 && $drop;
     }
     @x2_dim = (1) unless @x2_dim;
@@ -217,13 +228,72 @@ sub parse_index {
   }
 }
 
-# XS functions
-# index_to_pos()
-# pos_to_index()
-# cross_product()
+sub cross_product {
+  my $values = shift;
 
-=head1 NAME
+  my @idxs = (0) x @$values;
+  my @idx_idx = 0..(@idxs - 1);
+  my @x1 = map { $_->[0] } @$values;
+  my $result = [];
+  
+  push @$result, [@x1];
+  my $end_loop;
+  while (1) {
+    foreach my $i (@idx_idx) {
+      if( $idxs[$i] < @{$values->[$i]} - 1 ) {
+        $x1[$i] = $values->[$i][++$idxs[$i]];
+        push @$result, [@x1];
+        last;
+      }
+      
+      if ($i == $idx_idx[-1]) {
+        $end_loop = 1;
+        last;
+      }
+      
+      $idxs[$i] = 0;
+      $x1[$i] = $values->[$i][0];
+    }
+    last if $end_loop;
+  }
+  
+  return $result;
+}
 
-Rstats::Util - Utility class
+sub index_to_pos {
+  my ($index, $dim_values) = @_;
+  
+  my $pos = 0;
+  for (my $i = 0; $i < @$dim_values; $i++) {
+    if ($i > 0) {
+      my $tmp = 1;
+      $tmp *= $dim_values->[$_] for (0 .. $i - 1);
+      $pos += $tmp * ($index->[$i] - 1);
+    }
+    else {
+      $pos += $index->[$i];
+    }
+  }
+  
+  return $pos - 1;
+}
+
+sub pos_to_index {
+  my ($pos, $dim) = @_;
+  
+  my $index = [];
+  my $before_dim_product = 1;
+  $before_dim_product *= $dim->[$_] for (0 .. @$dim - 1);
+  for (my $i = @{$dim} - 1; $i >= 0; $i--) {
+    my $dim_product = 1;
+    $dim_product *= $dim->[$_] for (0 .. $i - 1);
+    my $reminder = $pos % $before_dim_product;
+    my $quotient = int ($reminder / $dim_product);
+    unshift @$index, $quotient + 1;
+    $before_dim_product = $dim_product;
+  }
+  
+  return $index;
+}
 
 1;
